
# q-expansion of newform 882.5.b.d, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 5
poly_data = [9, 0, 8, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 5, 0, 1], 3], [[0, 77, 0, 7], 3], [[56, 0, 14, 0], 1]]

hecke_ring_character_values = [[785, [-1, 0, 0, 0]], [199, [1, 0, 0, 0]]]
aps_data = [[0, 2, 0, 0], [0, 0, 0, 0], [0, -13, 1, 0], [0, 0, 0, 0], [0, -1, -2, 0], [-118, 0, 0, -2], [0, 97, -7, 0], [10, 0, 0, -15], [0, 221, -2, 0], [0, 73, 14, 0], [-34, 0, 0, 1], [-1048, 0, 0, 30], [0, 1125, -57, 0], [2396, 0, 0, -6], [0, -1380, 42, 0], [0, -165, -168, 0], [0, -1612, 106, 0], [-1132, 0, 0, -23], [-486, 0, 0, -38], [0, -253, -314, 0], [5340, 0, 0, 83], [2578, 0, 0, -16], [0, 3178, 290, 0], [0, -759, -105, 0], [11208, 0, 0, -91], [0, -9037, -41, 0], [6874, 0, 0, -51], [0, 7455, 114, 0], [8316, 0, 0, -332], [0, 5923, 344, 0], [14050, 0, 0, 122], [0, -4216, -476, 0], [0, -9167, 362, 0], [27648, 0, 0, -52], [0, 11289, 828, 0], [-12996, 0, 0, 850], [5020, 0, 0, 255], [19082, 0, 0, 342], [0, 11504, -410, 0], [0, -25243, 289, 0], [0, -1605, -1446, 0], [45926, 0, 0, 400], [0, 3333, -1302, 0], [-34624, 0, 0, -1068], [0, 13697, 1432, 0], [-7088, 0, 0, 582], [38096, 0, 0, -672], [20200, 0, 0, 1686], [0, 26508, 1350, 0], [-17862, 0, 0, 842], [0, -11533, -1418, 0], [0, 62907, -534, 0], [22612, 0, 0, -1203], [0, 10578, 3552, 0], [0, -42321, -2223, 0], [0, -75783, 558, 0], [0, 54545, 967, 0], [-13870, 0, 0, -2147], [47618, 0, 0, -1764], [0, 24995, -1418, 0], [-74430, 0, 0, -223], [0, -34749, -2541, 0], [26094, 0, 0, -3067], [0, 23528, 706, 0], [-2106, 0, 0, 2894], [0, -10257, 3792, 0], [38144, 0, 0, -1212], [65536, 0, 0, -1414], [0, -23733, 5718, 0], [24528, 0, 0, 2465], [0, -16407, -1131, 0], [0, -44927, 602, 0], [-26444, 0, 0, 2310], [31806, 0, 0, -1640], [21960, 0, 0, -5264], [0, 2836, -2560, 0], [0, 23395, -7234, 0], [217796, 0, 0, -1295], [0, -34419, -7878, 0], [-39632, 0, 0, 5391], [0, -11022, 588, 0], [-156098, 0, 0, -1480], [0, -14607, -2034, 0], [-182774, 0, 0, -2574], [123920, 0, 0, 5572], [0, -10513, -218, 0], [0, -108641, -3148, 0], [-141696, 0, 0, 2848], [0, -28087, -6155, 0], [24246, 0, 0, -4430], [0, -81802, -9116, 0], [0, -151322, -1648, 0], [292604, 0, 0, 2334], [0, -5463, -1050, 0], [-148596, 0, 0, -3338], [0, -107790, -5538, 0], [0, -29163, 963, 0], [0, -168951, 5565, 0], [218104, 0, 0, -2124], [-193430, 0, 0, 3224], [-154766, 0, 0, 8420], [0, -54515, 15428, 0], [0, 243472, -6586, 0], [0, -151041, -11682, 0], [152014, 0, 0, -3604], [-11502, 0, 0, -16858], [0, -111008, -1498, 0], [0, 283113, 7641, 0], [0, -160683, -7326, 0], [-285722, 0, 0, -8298], [-221908, 0, 0, 5782], [-9434, 0, 0, -718], [0, 54659, -15110, 0], [-2520, 0, 0, 1274], [-567022, 0, 0, 1974], [0, 49739, -5066, 0], [217210, 0, 0, 9261], [0, 43896, -5142, 0], [0, -34959, -21090, 0], [0, 8741, -3146, 0], [-48696, 0, 0, 13451], [-25492, 0, 0, -7428], [0, -44103, -16923, 0], [0, 447753, -6078, 0], [347344, 0, 0, 7170], [0, -108119, 10226, 0], [-165700, 0, 0, 216], [0, 158904, 19980, 0], [251730, 0, 0, -10291], [-210538, 0, 0, 6460], [-12290, 0, 0, -4546], [0, 250941, -2718, 0], [47416, 0, 0, 10148], [294412, 0, 0, 7568], [0, 287107, 1421, 0], [937818, 0, 0, 4736], [0, 17489, -23735, 0], [216740, 0, 0, -23012], [0, -192099, -5607, 0], [0, 479917, -14356, 0], [-461544, 0, 0, -6010], [0, -127111, 19288, 0], [-145686, 0, 0, -11516], [0, 158517, 11106, 0], [110622, 0, 0, 8516], [0, 524578, 2096, 0], [1030552, 0, 0, 4227], [0, 274959, 309, 0], [-1221794, 0, 0, -831], [0, 588433, 21650, 0], [958146, 0, 0, 7294], [0, -111915, -14859, 0], [-173068, 0, 0, -7254], [0, -201384, -26130, 0], [139060, 0, 0, -15814], [0, 581007, 9570, 0], [-990636, 0, 0, 12658], [0, 52035, 24621, 0], [997966, 0, 0, -5352], [0, -530737, -29327, 0], [0, 569635, -25138, 0], [0, -647875, -24308, 0], [372350, 0, 0, 39102], [0, 961986, -2658, 0], [0, -826113, 10158, 0], [0, -44682, 46566, 0], [408764, 0, 0, -36270], [-41420, 0, 0, 27087]]
